@@ -24,14 +24,14 @@ class DataConverterAdaptor:
         self._default_file_type = default_file_type
 
     async def convert_to_pandas(self, file: Path, file_type: Optional[str] = None):
-        """Convert to a pandas dataframe from data stored in a file of a particular file_type
+        """Convert to a pandas DataFrame from data stored in a file of a particular file_type
 
         Args:
             file (Path): Path to the file
             file_type (str): What the file contains (root, parquet, etc)
         """
         file_type = file_type if file_type is not None else self._default_file_type
-        if file_type == "root":
+        if file_type == "root-file":
             return await self._convert_root_to_pandas(file)
         elif file_type == "parquet":
             return await self._convert_parquet_to_pandas(file)
@@ -48,7 +48,7 @@ class DataConverterAdaptor:
             file_type (str): What the file contains (root, parquet, etc)
         """
         file_type = file_type if file_type is not None else self._default_file_type
-        if file_type == "root":
+        if file_type == "root-file":
             return await self._convert_root_to_awkward(file)
         elif file_type == "parquet":
             return await self._convert_parquet_to_awkward(file)
@@ -59,10 +59,10 @@ class DataConverterAdaptor:
             )
 
     def combine_pandas(self, dfs: Iterable[pd.DataFrame]) -> pd.DataFrame:
-        """Combine many pandas dataframes into a single one, in order.
+        """Combine many pandas DataFrame into a single one, in order.
 
         Args:
-            dfs (Iterable[pd.DataFrame]): The list of dataframes
+            dfs (Iterable[pd.DataFrame]): The list of DataFrames
         """
         return pd.concat(dfs)
 
@@ -84,7 +84,7 @@ class DataConverterAdaptor:
 
         Returns:
 
-            DataFrame   A pandas dataframe
+            DataFrame   A pandas DataFrame
 
         Note:
 
@@ -113,7 +113,7 @@ class DataConverterAdaptor:
 
         Returns:
 
-            DataFrame   A pandas dataframe
+            DataFrame   A pandas DataFrame
 
         Note:
 
@@ -138,14 +138,14 @@ class DataConverterAdaptor:
 
         Returns:
 
-            DataFrame   A pandas dataframe
+            DataFrame   A pandas DataFrame
 
         Note:
 
             - Work is done on a second thread.
             - Awkward is only imported if this is called.
             - A LazyArray is returned, so it isn't completely loaded into memory. That also means
-              this will leak filehandles - as that has to be left open.
+              this will leak file handles - as that has to be left open.
 
         """
 
@@ -169,7 +169,7 @@ class DataConverterAdaptor:
 
         Returns:
 
-            DataFrame   A pandas dataframe
+            DataFrame   A pandas DataFrame
 
         Note:
 
