@@ -22,6 +22,7 @@ import aiohttp
 import backoff
 import minio
 from backoff import on_exception
+from make_it_sync import make_sync
 
 from servicex.servicex_config import ServiceXConfigAdaptor
 
@@ -40,6 +41,7 @@ from .utils import (
     ServiceXException,
     ServiceXFailedFileTransform,
     ServiceXFatalTransformException,
+    ServiceXNoFilesInCache,
     ServiceXUnknownDataRequestID,
     ServiceXUnknownRequestID,
     StatusUpdateFactory,
@@ -53,6 +55,10 @@ from .utils import (
     stream_unique_updates_only,
 )
 
+# The allowed file formats.
+# You could modify this if you wanted to add new...
+g_allowed_formats = ["parquet", "root-file"]
+
 
 class StreamInfoBase:
     """Contains base information about results that are streamed back from
@@ -66,7 +72,7 @@ class StreamInfoBase:
     def file(self) -> str:
         """Returns the ServiceX filename
 
-        This filename is unique in the dataset, and will be the same accross different queries
+        This filename is unique in the dataset, and will be the same across different queries
         against the dataset. It can be used as a key to sort results.
 
         Notes:
@@ -100,7 +106,7 @@ class StreamInfoUrl(StreamInfoBase):
 
     @property
     def bucket(self) -> str:
-        """Returns the buck name - unique and constant accross transformations.
+        """Returns the buck name - unique and constant across transformations.
         Can be used to order the results
 
         Returns:
@@ -160,7 +166,7 @@ class StreamInfoData(StreamInfoBase):
 
 class ServiceXDataset(ServiceXABC):
     """
-    Used to access an instance of ServiceX at an end point on the internet. Support convieration
+    Used to access an instance of ServiceX at an end point on the internet. Support conversion
     by configuration object `config_adaptor` or by creating the adaptors defined in the `__init__`
     function.
     """
@@ -169,6 +175,8 @@ class ServiceXDataset(ServiceXABC):
         self,
         dataset: DatasetType,
         backend_name: Optional[str] = None,
+        backend_type: Optional[str] = None,
+        codegen: Optional[str] = None,
         image: Optional[str] = None,
         max_workers: int = 20,
         result_destination: str = "object-store",
@@ -195,6 +203,13 @@ class ServiceXDataset(ServiceXABC):
                                         will default to xaod, unless you have any endpoint listed
                                         in your servicex file. It will default to best match there,
                                         or fail if a name has been given.
+            backend_type                The type of backend. Overrides the `type` in the `yaml`
+                                        config file.
+            codegen                     The type of code generator passed to the backend to
+                                        generate the code that powers the requested transform.
+                                        Don't use unless you know what you are doing - sometimes
+                                        the return filetype is also needed. Better to use
+                                        `backend_type`!
             image                       Name of transformer image to use to transform the data. If
                                         left as default, `None`, then the default image for the
                                         ServiceX backend will be used.
@@ -204,7 +219,7 @@ class ServiceXDataset(ServiceXABC):
                                         Defaults to object-store, but could be used to save
                                         results to a posix volume
             servicex_adaptor            Object to control communication with the servicex instance
-                                        at a particular ip address with certian login credentials.
+                                        at a particular ip address with certain login credentials.
                                         Will be configured via the `config_adaptor` by default.
             minio_adaptor               Object to control communication with the minio servicex
                                         instance.
@@ -240,6 +255,7 @@ class ServiceXDataset(ServiceXABC):
         ServiceXABC.__init__(
             self,
             dataset,
+            codegen,
             image,
             max_workers,
             result_destination,
@@ -256,7 +272,9 @@ class ServiceXDataset(ServiceXABC):
             Cache(
                 get_configured_cache_path(config.settings),
                 ignore_cache,
-                analysis_query_key=backend_name if backend_name is not None else "default",
+                analysis_query_key=backend_name
+                if backend_name is not None
+                else "default",
             )
             if cache_adaptor is None
             else cache_adaptor
@@ -264,7 +282,9 @@ class ServiceXDataset(ServiceXABC):
 
         if not servicex_adaptor:
             # Given servicex adaptor is none, this should be ok. Fixes type checkers
-            end_point, token = config.get_servicex_adaptor_config(backend_name)
+            end_point, token = config.get_servicex_adaptor_config(
+                backend_name, backend_type=backend_type
+            )
             servicex_adaptor = ServiceXAdaptor(end_point, token)
         self._servicex_adaptor = servicex_adaptor
 
@@ -284,13 +304,24 @@ class ServiceXDataset(ServiceXABC):
             else default_client_session
         )
 
-        self._return_types = [config.get_default_returned_datatype(backend_name)]
+        self._return_types = [
+            config.get_default_returned_datatype(
+                backend_name, backend_type=backend_type
+            )
+        ]
         self._converter = (
             data_convert_adaptor
             if data_convert_adaptor is not None
             else DataConverterAdaptor(self._return_types[0])
         )
 
+        # TODO - this should not be in the ABC backend since we have to have some intelligence
+        # in setting it, it turns out.
+        if self._codegen is None:
+            self._codegen = config.get_backend_info(
+                backend_name, "codegen", backend_type=backend_type
+            )
+
     def first_supported_datatype(
         self, datatypes: Union[List[str], str]
     ) -> Optional[str]:
@@ -316,8 +347,8 @@ class ServiceXDataset(ServiceXABC):
         return None
 
     def ignore_cache(self):
-        """Return a context manager that, as long as it is held, will cause any queries against just
-        this dataset to ignore any locally cached data.
+        """Return a context manager that, as long as it is held, will cause any queries against
+        just this dataset to ignore any locally cached data.
 
         Returns:
             ContextManager: As long as this is held, the local query cache will be ignored.
@@ -339,7 +370,7 @@ class ServiceXDataset(ServiceXABC):
         access the data.
 
         Args:
-            selection_query (str): The `qastle` query for the data to retreive.
+            selection_query (str): The `qastle` query for the data to retrieve.
 
         Yields:
             AsyncIterator[StreamInfoPath]: As ServiceX completes the data, and it is downloaded
@@ -348,7 +379,7 @@ class ServiceXDataset(ServiceXABC):
                                            file locally.
         """
         async for f_info in self._stream_local_files(
-            selection_query, title, "root-files"
+            selection_query, title, "root-file"
         ):  # type: ignore
             yield f_info
 
@@ -367,7 +398,7 @@ class ServiceXDataset(ServiceXABC):
         access the data.
 
         Args:
-            selection_query (str): The `qastle` query for the data to retreive.
+            selection_query (str): The `qastle` query for the data to retrieve.
 
         Yields:
             AsyncIterator[StreamInfoPath]: As ServiceX completes the data, and it is downloaded
@@ -385,9 +416,13 @@ class ServiceXDataset(ServiceXABC):
     async def get_data_pandas_df_async(
         self, selection_query: str, title: Optional[str] = None
     ):
+        data_format = self._return_types[0]
         return self._converter.combine_pandas(
             await self._data_return(
-                selection_query, lambda f: self._converter.convert_to_pandas(f), title
+                selection_query,
+                lambda f: self._converter.convert_to_pandas(f),
+                title,
+                data_format=data_format,
             )
         )
 
@@ -396,9 +431,13 @@ class ServiceXDataset(ServiceXABC):
     async def get_data_awkward_async(
         self, selection_query: str, title: Optional[str] = None
     ):
+        data_format = self._return_types[0]
         return self._converter.combine_awkward(
             await self._data_return(
-                selection_query, lambda f: self._converter.convert_to_awkward(f), title
+                selection_query,
+                lambda f: self._converter.convert_to_awkward(f),
+                title,
+                data_format=data_format,
             )
         )
 
@@ -409,7 +448,7 @@ class ServiceXDataset(ServiceXABC):
         as a separate `awkward` array. The data is returned in a `StreamInfoData` object.
 
         Args:
-            selection_query (str): The `qastle` query for the data to retreive.
+            selection_query (str): The `qastle` query for the data to retrieve.
 
         Yields:
             AsyncIterator[StreamInfoData]: As ServiceX completes the data, and it is downloaded
@@ -429,7 +468,7 @@ class ServiceXDataset(ServiceXABC):
         as a separate `pandas.DataFrame` array. The data is returned in a `StreamInfoData` object.
 
         Args:
-            selection_query (str): The `qastle` query for the data to retreive.
+            selection_query (str): The `qastle` query for the data to retrieve.
 
         Yields:
             AsyncIterator[StreamInfoData]: As ServiceX completes the data, and it is downloaded
@@ -442,8 +481,11 @@ class ServiceXDataset(ServiceXABC):
         ):
             yield a
 
-    async def get_data_rootfiles_url_stream(
-        self, selection_query: str, title: Optional[str] = None
+    async def get_data_rootfiles_uri_stream(
+        self,
+        selection_query: str,
+        title: Optional[str] = None,
+        as_signed_url: Optional[bool] = False,
     ) -> AsyncIterator[StreamInfoUrl]:
         """Returns, as an async iterator, each completed batch of work from ServiceX.
         The data that comes back includes a `url` that can be accessed to download the
@@ -451,26 +493,76 @@ class ServiceXDataset(ServiceXABC):
 
         Args:
             selection_query (str): The ServiceX Selection
+            title (str): Optional title for transform request
+            as_signed_url (bool): Return the uri as a presigned http url?
         """
         async for f_info in self._stream_url_buckets(
-            selection_query, "root-files", title
+            selection_query, "root-file", title, as_signed_url
         ):  # type: ignore
             yield f_info
 
-    async def get_data_parquet_url_stream(
-        self, selection_query: str, title: Optional[str] = None
+    async def get_data_rootfiles_uri_async(
+        self,
+        selection_query: str,
+        title: Optional[str] = None,
+        as_signed_url: Optional[bool] = False,
+    ) -> List[StreamInfoUrl]:
+        """Returns a list of StreamInfoUrl entries containing a `url` for each output file
+        from the transform, taken from get_data_rootfiles_uri_stream.
+        The data that comes back includes a `url` that can be accessed to download the
+        data.
+
+        Args:
+            selection_query (str): The ServiceX Selection
+            title (str): Optional title for transform request
+            as_signed_url (bool): Return the uri as a presigned http url?
+        """
+        return [
+            f
+            async for f in self.get_data_rootfiles_uri_stream(
+                selection_query, title=title, as_signed_url=as_signed_url
+            )
+        ]
+
+    async def get_data_parquet_uri_stream(
+        self,
+        selection_query: str,
+        title: Optional[str] = None,
+        as_signed_url: Optional[bool] = False,
     ) -> AsyncIterator[StreamInfoUrl]:
         """Returns, as an async iterator, each of the files from the minio bucket,
         as the files are added there.
 
         Args:
             selection_query (str): The ServiceX Selection
+            title (str): Optional title for transform request
+            as_signed_url (bool): Return the uri as a presigned http url?
         """
         async for f_info in self._stream_url_buckets(
-            selection_query, "parquet", title
+            selection_query, "parquet", title, as_signed_url
         ):  # type: ignore
             yield f_info
 
+    async def get_data_parquet_uri_async(
+        self,
+        selection_query: str,
+        title: Optional[str] = None,
+        as_signed_url: Optional[bool] = False,
+    ) -> List[StreamInfoUrl]:
+        """Returns a list of each of the files from the minio bucket.
+
+        Args:
+            selection_query (str): The ServiceX Selection
+            title (str): Optional title for transform request
+            as_signed_url (bool): Return the uri as a presigned http url?
+        """
+        return [
+            f_info
+            async for f_info in self.get_data_parquet_uri_stream(
+                selection_query, title, as_signed_url
+            )
+        ]
+
     async def _file_return(
         self, selection_query: str, data_format: str, title: Optional[str]
     ):
@@ -478,7 +570,7 @@ class ServiceXDataset(ServiceXABC):
         Given a query, return the list of files, in a unique order, that hold
         the data for the query.
 
-        For certian types of exceptions, the queries will be repeated. For example,
+        For certain types of exceptions, the queries will be repeated. For example,
         if `ServiceX` indicates that it was restarted in the middle of the query, then
         the query will be re-submitted.
 
@@ -511,7 +603,11 @@ class ServiceXDataset(ServiceXABC):
         max_tries=2,
     )
     async def _stream_url_buckets(
-        self, selection_query: str, data_format: str, title: Optional[str]
+        self,
+        selection_query: str,
+        data_format: str,
+        title: Optional[str],
+        as_signed_url: Optional[bool],
     ) -> AsyncGenerator[StreamInfoUrl, None]:
         """Get a list of files back for a request
 
@@ -519,6 +615,7 @@ class ServiceXDataset(ServiceXABC):
             selection_query (str): The selection query we are to do
             data_format (str): The requested file format
             title (Optional[str]): The title of transform to pass to ServiceX
+            as_signed_url (Optional[bool]): Return the uri as a presigned http url?
 
         Yields:
             AsyncIterator[Dict[str, str]]: A tuple of the minio bucket and file in that bucket.
@@ -529,7 +626,6 @@ class ServiceXDataset(ServiceXABC):
         query = self._build_json_query(selection_query, data_format, title)
 
         async with aiohttp.ClientSession() as client:
-
             # Get a request id - which might be cached, but if not, submit it.
             request_id = await self._get_request_id(client, query)
 
@@ -542,7 +638,7 @@ class ServiceXDataset(ServiceXABC):
                 self._cache.lookup_query_status(request_id)
             )
 
-            # Look up the cache, and then fetch an iterator going thorugh the results
+            # Look up the cache, and then fetch an iterator going through the results
             # from either servicex or the cache, depending.
             try:
                 notifier = self._create_notifier(title, False)
@@ -551,11 +647,15 @@ class ServiceXDataset(ServiceXABC):
                 )
 
                 # Reflect the files back up a level.
-                async for r in minio_files:
-                    yield StreamInfoUrl(
-                        r, minio_adaptor.get_access_url(request_id, r), request_id
+                async for object_name in minio_files:
+                    uri = (
+                        minio_adaptor.get_access_url(request_id, object_name)
+                        if as_signed_url
+                        else minio_adaptor.get_s3_uri(request_id, object_name)
                     )
 
+                    yield StreamInfoUrl(object_name, uri, request_id)
+
                 # Cache the final status
                 await self._update_query_status(client, request_id)
 
@@ -601,7 +701,7 @@ class ServiceXDataset(ServiceXABC):
         """Given a query, return the data, in a unique order, that hold
         the data for the query.
 
-        For certian types of exceptions, the queries will be repeated. For example,
+        For certain types of exceptions, the queries will be repeated. For example,
         if `ServiceX` indicates that it was restarted in the middle of the query, then
         the query will be re-submitted.
 
@@ -641,7 +741,7 @@ class ServiceXDataset(ServiceXABC):
         """Given a query, return the data, in the order it arrives back
         converted as appropriate.
 
-        For certian types of exceptions, the queries will be repeated. For example,
+        For certain types of exceptions, the queries will be repeated. For example,
         if `ServiceX` indicates that it was restarted in the middle of the query, then
         the query will be re-submitted.
 
@@ -672,7 +772,7 @@ class ServiceXDataset(ServiceXABC):
         that contain the results of the query. This is an async generator, and files
         are returned as they arrive.
 
-        For certian types of exceptions, the queries will be repeated. For example,
+        For certain types of exceptions, the queries will be repeated. For example,
         if `ServiceX` indicates that it was restarted in the middle of the query, then
         the query will be re-submitted.
 
@@ -702,7 +802,7 @@ class ServiceXDataset(ServiceXABC):
     async def _get_files(
         self,
         selection_query: str,
-        data_type: str,
+        data_format: str,
         notifier: _status_update_wrapper,
         title: Optional[str],
     ) -> AsyncIterator[Tuple[str, Awaitable[Path]]]:
@@ -710,7 +810,7 @@ class ServiceXDataset(ServiceXABC):
         Return a list of files from servicex as they have been downloaded to this machine. The
         return type is an awaitable that will yield the path to the file.
 
-        For certian types of `ServiceX` failures we will automatically attempt a few retries:
+        For certain types of `ServiceX` failures we will automatically attempt a few retries:
 
             - When `ServiceX` forgets the query. This sometimes happens when a user submits a
               query, and then disconnects from the network, `ServiceX` is restarted, and then the
@@ -721,7 +821,7 @@ class ServiceXDataset(ServiceXABC):
         Arguments:
 
             selection_query             The query string to send to ServiceX
-            data_type                   The type of data that we want to come back.
+            data_format                 The type of data that we want to come back.
             notifier                    Status callback to let our progress be advertised
             title                       Title to pass to servicex backend.
 
@@ -731,10 +831,9 @@ class ServiceXDataset(ServiceXABC):
                                         This is returned this way so a number of downloads can run
                                         simultaneously.
         """
-        query = self._build_json_query(selection_query, data_type, title)
+        query = self._build_json_query(selection_query, data_format, title)
 
         async with aiohttp.ClientSession() as client:
-
             # Get a request id - which might be cached, but if not, submit it.
             request_id = await self._get_request_id(client, query)
 
@@ -747,7 +846,7 @@ class ServiceXDataset(ServiceXABC):
                 self._cache.lookup_query_status(request_id)
             )
 
-            # Look up the cache, and then fetch an iterator going thorugh the results
+            # Look up the cache, and then fetch an iterator going through the results
             # from either servicex or the cache, depending.
             try:
                 cached_files = self._cache.lookup_files(request_id)
@@ -775,6 +874,14 @@ class ServiceXDataset(ServiceXABC):
                     " cache. Please resubmit to trigger a new "
                     "query."
                 ) from e
+            except ServiceXNoFilesInCache as e:
+                self._cache.remove_query(query)
+                raise ServiceXUnknownDataRequestID(
+                    "Expected the ServiceX backend to have cached data "
+                    f"from query {request_id}. It did not. Cleared local"
+                    " cache. Please resubmit to trigger a new "
+                    "query."
+                ) from e
 
             except ServiceXFatalTransformException as e:
                 transform_status = await self._servicex_adaptor.get_query_status(
@@ -876,7 +983,6 @@ class ServiceXDataset(ServiceXABC):
         start_time = time.monotonic()
         good = True
         try:
-
             # Get the stream of minio bucket new files.
             stream_new_object = self._get_minio_bucket_files_from_servicex(
                 request_id, client, minio_adaptor, notifier
@@ -933,7 +1039,6 @@ class ServiceXDataset(ServiceXABC):
         """
         start_time = time.monotonic()
         try:
-
             # Setup the status sequence from servicex
             stream_status = transform_status_stream(
                 self._servicex_adaptor, client, request_id
@@ -960,25 +1065,30 @@ class ServiceXDataset(ServiceXABC):
             )
 
     def _build_json_query(
-        self, selection_query: str, data_type: str, title: Optional[str]
+        self, selection_query: str, data_format: str, title: Optional[str]
     ) -> Dict[str, Union[str, Iterable[str]]]:
         """
         Returns a list of locally written files for a given selection query.
 
         Arguments:
             selection_query         The query to be send into the ServiceX API
-            data_type               What is the output data type (parquet, root-file, etc.)
+            data_format             What is the output data type (parquet, root-file, etc.)
 
         Notes:
             - Internal routine.
         """
+        assert data_format in g_allowed_formats
+
         # Items that must always be present
+        codegen = self._codegen if self._codegen is not None else "default"
+
         json_query: Dict[str, Union[str, Iterable[str]]] = {
             "selection": selection_query,
             "result-destination": self._result_destination,
-            "result-format": "parquet" if data_type == "parquet" else "root-file",
+            "result-format": data_format,
             "chunk-size": "1000",
             "workers": str(self._max_workers),
+            "codegen": codegen,
         }
 
         # Add the appropriate did.
@@ -1003,3 +1113,8 @@ class ServiceXDataset(ServiceXABC):
         )
 
         return json_query
+
+    # Define the synchronous versions of the async methods for easy of use
+
+    get_data_rootfiles_uri = make_sync(get_data_rootfiles_uri_async)
+    get_data_parquet_uri = make_sync(get_data_parquet_uri_async)
