@@ -90,6 +90,16 @@ class ServiceXUnknownRequestID(Exception):
         super().__init__(self, msg)
 
 
+class ServiceXNoFilesInCache(Exception):
+    """Raised when we try to access ServiceX with a request ID that it
+    knows about, but no longer has the results cached - and they
+    need to be regenerated"
+    """
+
+    def __init__(self, msg):
+        super().__init__(self, msg)
+
+
 class ServiceXUnknownDataRequestID(Exception):
     """Raised when we expect ServiceX to know about the data for a request id, but it does not.
     Possible causes:
@@ -340,7 +350,7 @@ class _default_wrapper_mgr:
     def _update_bar(
         self, bar: Optional[tqdm], total: Optional[int], num: int, failed: int
     ):
-        assert bar is not None, "Internal error - bar was not initalized"
+        assert bar is not None, "Internal error - bar was not initialized"
         if total is not None:
             if bar.total != total:
                 # There is a bug in the tqm library if running in a notebook
@@ -407,7 +417,7 @@ def clean_linq(q: str) -> str:
 
     Arguments
 
-        q           Strign containing the qastle code`
+        q           String containing the qastle code`
 
     Returns
 
@@ -579,7 +589,13 @@ def retry_exception_itr(
         while True:
             tries += 1
             elapsed = timedelta.total_seconds(datetime.datetime.now() - start)
-            details = (target, args, kwargs, tries, elapsed)
+            details = {
+                "target": target,
+                "args": args,
+                "kwargs": kwargs,
+                "tries": tries,
+                "elapsed": elapsed,
+            }
 
             got_one_item = False
             try:
@@ -598,16 +614,18 @@ def retry_exception_itr(
                 max_time_exceeded = max_time_ is not None and elapsed >= max_time_
 
                 if giveup_result or max_tries_exceeded or max_time_exceeded:
-                    await _call_handlers(on_giveup, *details)
+                    await _call_handlers(on_giveup, **details)
                     raise
 
                 try:
-                    seconds = _next_wait(wait, jitter, elapsed, max_time_)
+                    seconds = _next_wait(
+                        wait, giveup_result, jitter, elapsed, max_time_
+                    )
                 except StopIteration:  # pragma: no cover
-                    await _call_handlers(on_giveup, *details)
+                    await _call_handlers(on_giveup, **details)
                     raise e
 
-                await _call_handlers(on_backoff, *details, wait=seconds)
+                await _call_handlers(on_backoff, **details, wait=seconds)
 
                 # Note: there is no convenient way to pass explicit event
                 # loop to decorator, so here we assume that either default
@@ -620,7 +638,7 @@ def retry_exception_itr(
                 #   <https://bugs.python.org/issue28613>
                 await asyncio.sleep(seconds)
             else:
-                await _call_handlers(on_success, *details)
+                await _call_handlers(on_success, **details)
                 return
 
     return retry
@@ -647,9 +665,17 @@ def on_exception_itr(
 
         on_success_ = _config_handlers(on_success)
         on_backoff_ = _config_handlers(
-            on_backoff, _log_backoff, logger_, backoff_log_level
+            on_backoff,
+            default_handler=_log_backoff,
+            logger=logger_,
+            log_level=backoff_log_level,
+        )
+        on_giveup_ = _config_handlers(
+            on_giveup,
+            default_handler=_log_giveup,
+            logger=logger_,
+            log_level=giveup_log_level,
         )
-        on_giveup_ = _config_handlers(on_giveup, _log_giveup, logger_, giveup_log_level)
 
         return retry_exception_itr(
             target,
